# -*- coding: utf-8 -*-
import hashlib
import hmac
import json
import os
import time
import urllib.parse
from typing import Dict, Generator, List, Tuple

from PIL import Image  # type: ignore

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils import http
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ot, ct
from jarvis.jarvis_utils.utils import while_success


class YuanbaoPlatform(BasePlatform):
    """Hunyuan模型实现"""

    def get_model_list(self) -> List[Tuple[str, str]]:
        """获取支持的模型列表"""
        return [
            ("deep_seek", "DeepSeek-R1"),
            ("deep_seek_v3", "DeepSeek-v3"),
            ("hunyuan_gpt_175B_0404", "Tencent Hunyuan"),
            ("hunyuan_t1", "Tencent Hunyuan-T1"),
        ]

    def __init__(self):
        """
        初始化Hunyuan模型
        """
        super().__init__()
        self.conversation_id = ""  # 会话ID，用于标识当前对话
        # 从环境变量中获取必要参数
        self.cookies = os.getenv("YUANBAO_COOKIES")  # 认证cookies
        self.agent_id = "naQivTmsDa"

        if not self.cookies:
            raise ValueError(
                "YUANBAO_COOKIES environment variable not set. Please provide your cookies to use the Yuanbao platform."
            )

        self.system_message = ""  # 系统消息，用于初始化对话
        self.first_chat = True  # 标识是否为第一次对话
        self.model_name = "deep_seek_v3"  # 默认模型名称，使用下划线保持一致
        self.multimedia = []

    def set_system_prompt(self, message: str):
        """设置系统消息"""
        self.system_message = message

    def set_model_name(self, model_name: str):
        # 模型映射表，可以根据需要扩展
        model_mapping = [m[0] for m in self.get_model_list()]

        if model_name in model_mapping:
            self.model_name = model_name
        else:
            PrettyOutput.print(f"错误：不支持的模型: {model_name}", OutputType.ERROR)

    def _get_base_headers(self):
        """获取API请求的基础头部信息"""
        return {
            "Host": "yuanbao.tencent.com",
            "X-Language": "zh-CN",
            "X-Requested-With": "XMLHttpRequest",
            "chat_version": "v1",
            "X-Instance-ID": "5",
            "X-Requested-With": "XMLHttpRequest",
            "Accept": "application/json, text/plain, */*",
            "Content-Type": "application/json",
            "sec-ch-ua-mobile": "?0",
            "Origin": "https://yuanbao.tencent.com",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/133.0.0.0 Safari/537.36 Edg/133.0.0.0",
            "Referer": f"https://yuanbao.tencent.com/chat/{self.agent_id}",
            "X-Source": "web",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Accept": "*/*",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
            "Cookie": self.cookies,
        }

    def _create_conversation(self) -> bool:
        """创建一个新的对话会话"""
        url = "https://yuanbao.tencent.com/api/user/agent/conversation/create"

        headers = self._get_base_headers()

        payload = json.dumps({"agentId": self.agent_id})

        try:
            response = while_success(
                lambda: http.post(url, headers=headers, data=payload),
                sleep_time=5,
            )
            response_json = response.json()

            if "id" in response_json:
                self.conversation_id = response_json["id"]
                return True
            else:
                PrettyOutput.print(
                    f"错误：创建会话失败，响应: {response_json}", OutputType.ERROR
                )
                return False
        except Exception as e:
            PrettyOutput.print(f"错误：创建会话失败：{e}", OutputType.ERROR)
            return False

    def support_upload_files(self) -> bool:
        """Check if platform supports upload files"""
        return True

    def upload_files(self, file_list: List[str]) -> bool:
        """上传文件到元宝平台

        参数:
            file_list: 要上传的文件路径列表

        返回:
            用于聊天消息的文件元数据字典列表
        """
        if not self.cookies:
            PrettyOutput.print("未设置YUANBAO_COOKIES，无法上传文件", OutputType.ERROR)
            return False

        uploaded_files = []

        for file_path in file_list:
            file_name = os.path.basename(file_path)
            print(f"🔍 上传文件 {file_name}")
            try:
                # 1. Prepare the file information
                print(f"🔍 准备文件信息: {file_name}")
                file_size = os.path.getsize(file_path)
                file_extension = os.path.splitext(file_path)[1].lower().lstrip(".")

                # Determine file_type using file extension
                file_type = "txt"  # Default type

                # Image types
                if file_extension in ["jpg", "jpeg", "png", "webp", "bmp", "gif"]:
                    file_type = "image"
                # PDF type
                elif file_extension == "pdf":
                    file_type = "pdf"
                # Spreadsheet types
                elif file_extension in ["xls", "xlsx"]:
                    file_type = "excel"
                # Presentation types
                elif file_extension in ["ppt", "pptx"]:
                    file_type = "ppt"
                # Document types
                elif file_extension in ["doc", "docx"]:
                    file_type = "doc"
                # Code file types
                elif file_extension in [
                    "bat",
                    "c",
                    "cpp",
                    "cs",
                    "css",
                    "go",
                    "h",
                    "hpp",
                    "ini",
                    "java",
                    "js",
                    "json",
                    "log",
                    "lua",
                    "php",
                    "pl",
                    "py",
                    "rb",
                    "sh",
                    "sql",
                    "swift",
                    "tex",
                    "toml",
                    "vue",
                    "yaml",
                    "yml",
                    "rs",
                ]:
                    file_type = "code"

                # 2. Generate upload information
                print(f"🔍 获取上传信息: {file_name}")
                upload_info = self._generate_upload_info(file_name)
                if not upload_info:
                    print(f"❌ 无法获取文件 {file_name} 的上传信息")
                    return False

                # 3. Upload the file to COS
                print(f"🔍 上传文件到云存储: {file_name}")
                upload_success = self._upload_file_to_cos(file_path, upload_info)
                if not upload_success:
                    print(f"❌ 上传文件 {file_name} 失败")
                    return False

                # 4. Create file metadata for chat
                print(f"🔍 生成文件元数据: {file_name}")
                file_metadata = {
                    "type": file_type,
                    "docType": file_extension if file_extension else file_type,
                    "url": upload_info.get("resourceUrl", ""),
                    "fileName": file_name,
                    "size": file_size,
                    "width": 0,
                    "height": 0,
                }

                # Get image dimensions if it's an image file
                if file_type == "image":
                    try:
                        with Image.open(file_path) as img:
                            file_metadata["width"] = img.width
                            file_metadata["height"] = img.height
                    except Exception as e:
                        print(f"⚠️ 无法获取图片 {file_name} 的尺寸: {str(e)}")

                uploaded_files.append(file_metadata)
                print(f"✅ 文件 {file_name} 上传成功")
                time.sleep(3)  # 上传成功后等待3秒

            except Exception as e:
                print(f"❌ 上传文件 {file_path} 时出错: {str(e)}")
                return False

        self.multimedia = uploaded_files
        return True

    def _generate_upload_info(self, file_name: str) -> Dict:
        """从元宝API生成上传信息

        参数:
            file_name: 要上传的文件名

        返回:
            包含上传信息的字典，如果失败则返回空字典
        """
        url = "https://yuanbao.tencent.com/api/resource/genUploadInfo"

        headers = self._get_base_headers()

        payload = {"fileName": file_name, "docFrom": "localDoc", "docOpenId": ""}

        try:
            response = while_success(
                lambda: http.post(url, headers=headers, json=payload),
                sleep_time=5,
            )

            if response.status_code != 200:
                PrettyOutput.print(
                    f"获取上传信息失败，状态码: {response.status_code}",
                    OutputType.ERROR,
                )
                if hasattr(response, "text"):
                    PrettyOutput.print(f"响应: {response.text}", OutputType.ERROR)
                return {}

            upload_info = response.json()
            return upload_info

        except Exception as e:
            PrettyOutput.print(f"获取上传信息时出错: {str(e)}", OutputType.ERROR)
            return {}

    def _upload_file_to_cos(self, file_path: str, upload_info: Dict) -> bool:
        """使用提供的上传信息将文件上传到腾讯COS

        参数:
            file_path: 要上传的文件路径
            upload_info: 从generate_upload_info获取的上传信息

        返回:
            布尔值表示成功或失败
        """
        try:
            # Extract required information from upload_info
            bucket_url = f"https://{upload_info['bucketName']}.{upload_info.get('accelerateDomain', 'cos.accelerate.myqcloud.com')}"
            object_path = upload_info.get("location", "")
            url = f"{bucket_url}{object_path}"

            # Security credentials
            tmp_secret_id = upload_info.get("encryptTmpSecretId", "")
            tmp_secret_key = upload_info.get("encryptTmpSecretKey", "")
            token = upload_info.get("encryptToken", "")
            start_time = upload_info.get("startTime", int(time.time()))
            expired_time = upload_info.get("expiredTime", start_time + 600)
            key_time = f"{start_time};{expired_time}"

            # Read file content
            with open(file_path, "rb") as file:
                file_content = file.read()

            print(f"ℹ️  上传文件大小: {len(file_content)}")

            # Prepare headers for PUT request
            host = f"{upload_info['bucketName']}.{upload_info.get('accelerateDomain', 'cos.accelerate.myqcloud.com')}"
            headers = {
                "Host": host,
                "Content-Length": str(len(file_content)),
                "Content-Type": "application/octet-stream",
                "x-cos-security-token": token,
            }

            # Generate signature for COS request (per Tencent Cloud documentation)
            signature = self._generate_cos_signature(
                secret_key=tmp_secret_key,
                method="PUT",
                path=urllib.parse.quote(object_path),
                params={},
                headers={"host": host, "content-length": headers["Content-Length"]},
                key_time=key_time,
            )

            # Add Authorization header with signature
            headers["Authorization"] = (
                f"q-sign-algorithm=sha1&q-ak={tmp_secret_id}&q-sign-time={key_time}&"
                f"q-key-time={key_time}&q-header-list=content-length;host&"
                f"q-url-param-list=&q-signature={signature}"
            )

            # Upload the file
            response = http.put(url, headers=headers, data=file_content)

            if response.status_code not in [200, 204]:
                PrettyOutput.print(
                    f"文件上传到COS失败，状态码: {response.status_code}",
                    OutputType.ERROR,
                )
                if hasattr(response, "text"):
                    PrettyOutput.print(f"响应: {response.text}", OutputType.ERROR)
                return False

            return True

        except Exception as e:
            PrettyOutput.print(f"上传文件到COS时出错: {str(e)}", OutputType.ERROR)
            return False

    def _generate_cos_signature(
        self,
        secret_key: str,
        method: str,
        path: str,
        params: Dict,
        headers: Dict,
        key_time: str,
    ) -> str:
        """根据腾讯云COS文档生成COS签名

        参数:
            secret_key: 临时密钥
            method: HTTP方法(GET, PUT等)
            path: 对象路径
            params: URL参数
            headers: HTTP头部
            key_time: 签名时间范围

        返回:
            签名字符串
        """
        try:
            # 1. Generate SignKey
            sign_key = hmac.new(
                secret_key.encode("utf-8"), key_time.encode("utf-8"), hashlib.sha1
            ).hexdigest()

            # 2. Format parameters and headers
            formatted_params = "&".join(
                [
                    f"{k.lower()}={urllib.parse.quote(str(v), safe='')}"
                    for k, v in sorted(params.items())
                ]
            )

            formatted_headers = "&".join(
                [
                    f"{k.lower()}={urllib.parse.quote(str(v), safe='')}"
                    for k, v in sorted(headers.items())
                ]
            )

            # 3. Generate HttpString
            http_string = (
                f"{method.lower()}\n{path}\n{formatted_params}\n{formatted_headers}\n"
            )

            # 4. Generate StringToSign
            string_to_sign = f"sha1\n{key_time}\n{hashlib.sha1(http_string.encode('utf-8')).hexdigest()}\n"

            # 5. Generate Signature
            signature = hmac.new(
                sign_key.encode("utf-8"), string_to_sign.encode("utf-8"), hashlib.sha1
            ).hexdigest()

            return signature

        except Exception as e:
            PrettyOutput.print(f"生成签名时出错: {str(e)}", OutputType.ERROR)
            raise e

    def chat(self, message: str) -> Generator[str, None, None]:
        """发送消息并获取响应，可选文件附件

        参数:
            message: 要发送的消息文本

        返回:
            模型的响应
        """
        if not self.conversation_id:
            if not self._create_conversation():
                raise Exception("Failed to create conversation session")

        url = f"https://yuanbao.tencent.com/api/chat/{self.conversation_id}"

        headers = self._get_base_headers()

        chat_model_ext_info = {
            "modelId": self.model_name,
            "subModelId": "",
            "supportFunctions": (
                ["openInternetSearch"] if self.web else ["autoInternetSearch"]
            ),
        }

        # 准备消息内容
        payload = {
            "model": "gpt_175B_0404",
            "prompt": message,
            "plugin": "Adaptive",
            "displayPrompt": message,
            "displayPromptType": 1,
            "options": {
                "imageIntention": {
                    "needIntentionModel": True,
                    "backendUpdateFlag": 2,
                    "intentionStatus": True,
                }
            },
            "multimedia": self.multimedia,
            "agentId": self.agent_id,
            "supportHint": 1,
            "version": "v2",
            "supportFunctions": chat_model_ext_info["supportFunctions"],
            "chatModelId": self.model_name,
        }

        if self.first_chat:
            payload["chatModelExtInfo"] = (json.dumps(chat_model_ext_info),)

        self.multimedia = []

        # 添加系统消息（如果是第一次对话）
        if self.first_chat and self.system_message:
            payload["prompt"] = f"{self.system_message}\n\n{message}"
            payload["displayPrompt"] = payload["prompt"]

        try:
            # 使用新的stream_post接口发送消息请求，获取流式响应
            response_stream = while_success(
                lambda: http.stream_post(url, headers=headers, json=payload),
                sleep_time=5,
            )

            in_thinking = False

            # 处理流式响应
            for line in response_stream:
                if not line.strip():
                    continue

                # SSE格式的行通常以"data: "开头
                if line.startswith("data: "):
                    try:
                        data_str = line[6:]  # 移除"data: "前缀

                        # 检查结束标志
                        if data_str == "[DONE]":
                            self.first_chat = False
                            return

                        data = json.loads(data_str)

                        # 处理文本类型的消息
                        if data.get("type") == "text":
                            if in_thinking:
                                yield f"{ct('think')}\n"
                                in_thinking = False
                            msg = data.get("msg", "")
                            if msg:
                                yield msg

                        # 处理思考中的消息
                        elif data.get("type") == "think":
                            if not in_thinking:
                                yield f"{ot('think')}\n"
                                in_thinking = True
                            think_content = data.get("content", "")
                            if think_content:
                                yield think_content

                    except json.JSONDecodeError:
                        pass
                else:
                    try:
                        data = json.loads(line)
                        if "msg" in data:
                            yield data["msg"]
                    except json.JSONDecodeError:
                        pass

            self.first_chat = False
            return

        except Exception as e:
            raise Exception(f"对话失败: {str(e)}")

    def delete_chat(self) -> bool:
        """删除当前会话"""
        if not self.conversation_id:
            return True  # 如果没有会话ID，视为删除成功

        # Hunyuan使用专门的clear API来清除会话
        url = "https://yuanbao.tencent.com/api/user/agent/conversation/v1/clear"

        # 为这个请求获取基础头部
        headers = self._get_base_headers()

        # 更新X-AgentID头部，需要包含会话ID
        headers.update({"X-AgentID": f"{self.agent_id}/{self.conversation_id}"})

        # 创建请求体，包含要删除的会话ID
        payload = {"conversationIds": [self.conversation_id]}

        try:
            response = while_success(
                lambda: http.post(url, headers=headers, json=payload),
                sleep_time=5,
            )

            if response.status_code == 200:
                self.conversation_id = ""
                self.first_chat = True
                return True
            else:
                PrettyOutput.print(
                    f"删除会话失败: HTTP {response.status_code}", OutputType.WARNING
                )
                if hasattr(response, "text"):
                    PrettyOutput.print(f"响应: {response.text}", OutputType.WARNING)
                return False
        except Exception as e:
            PrettyOutput.print(f"删除会话时发生错误: {str(e)}", OutputType.ERROR)
            return False

    def save(self, file_path: str) -> bool:
        """Save chat session to a file."""
        if not self.conversation_id:
            PrettyOutput.print("没有活动的会话可供保存", OutputType.WARNING)
            return False

        state = {
            "conversation_id": self.conversation_id,
            "system_message": self.system_message,
            "first_chat": self.first_chat,
            "model_name": self.model_name,
            "multimedia": self.multimedia,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.print(f"会话已成功保存到 {file_path}", OutputType.SUCCESS)
            return True
        except Exception as e:
            PrettyOutput.print(f"保存会话失败: {str(e)}", OutputType.ERROR)
            return False

    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.conversation_id = state.get("conversation_id", "")
            self.system_message = state.get("system_message", "")
            self.first_chat = state.get("first_chat", True)
            self.model_name = state.get("model_name", "deep_seek_v3")
            self.multimedia = state.get("multimedia", [])
            self._saved = True

            PrettyOutput.print(f"从 {file_path} 成功恢复会话", OutputType.SUCCESS)
            return True
        except FileNotFoundError:
            PrettyOutput.print(f"会话文件未找到: {file_path}", OutputType.ERROR)
            return False
        except Exception as e:
            PrettyOutput.print(f"恢复会话失败: {str(e)}", OutputType.ERROR)
            return False

    def name(self) -> str:
        """模型名称"""
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """平台名称"""
        return "yuanbao"

    def support_web(self) -> bool:
        """Yuanbao平台支持web功能"""
        return True



# -*- coding: utf-8 -*-
import os
import re
import subprocess
import tempfile
import sys
from typing import Any, Dict, List, Optional

import typer

from jarvis.jarvis_agent import Agent
from jarvis.jarvis_code_analysis.checklists.loader import get_language_checklist
from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_tools.read_code import ReadCodeTool
from jarvis.jarvis_utils.globals import get_agent, current_agent_name
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot
from jarvis.jarvis_utils.utils import init_env, is_context_overflow

app = typer.Typer(help="Autonomous code review tool")


class CodeReviewTool:
    name = "code_review"
    description = "自动代码审查工具，用于分析代码变更"
    labels = ["code", "analysis", "review"]
    parameters = {
        "type": "object",
        "properties": {
            "review_type": {
                "type": "string",
                "description": "审查类型：'commit' 审查特定提交，'current' 审查当前变更，'range' 审查提交范围，'file' 审查特定文件",
                "enum": ["commit", "current", "range", "file"],
                "default": "current",
            },
            "commit_sha": {
                "type": "string",
                "description": "要分析的提交SHA（review_type='commit'时必填）",
            },
            "start_commit": {
                "type": "string",
                "description": "起始提交SHA（review_type='range'时必填）",
            },
            "end_commit": {
                "type": "string",
                "description": "结束提交SHA（review_type='range'时必填）",
            },
            "file_path": {
                "type": "string",
                "description": "要审查的文件路径（review_type='file'时必填）",
            },
            "root_dir": {
                "type": "string",
                "description": "代码库根目录路径（可选）",
                "default": ".",
            },
        },
        "required": [],
    }

    def _detect_languages_from_files(self, file_paths: List[str]) -> List[str]:
        """
        Detect programming languages from a list of file paths using file extensions.
        Returns a list of detected languages ('c_cpp', 'go', 'python', 'rust', 'java', 'javascript', 'typescript', etc.).
        """
        if not file_paths:
            return []

        # Extension-based language detection
        languages = set()
        for file_path in file_paths:
            file_path = file_path.lower()
            _, ext = os.path.splitext(file_path)

            # Get base name for special files without extensions
            base_name = os.path.basename(file_path)

            # C/C++
            if ext in [
                ".c",
                ".cpp",
                ".cc",
                ".cxx",
                ".h",
                ".hpp",
                ".hxx",
                ".inl",
                ".ipp",
            ]:
                languages.add("c_cpp")

            # Go
            elif ext in [".go"]:
                languages.add("go")

            # Python
            elif ext in [".py", ".pyw", ".pyi", ".pyx", ".pxd"] or base_name in [
                "requirements.txt",
                "setup.py",
                "pyproject.toml",
            ]:
                languages.add("python")

            # Rust
            elif ext in [".rs", ".rlib"] or base_name in ["Cargo.toml", "Cargo.lock"]:
                languages.add("rust")

            # Java
            elif ext in [".java", ".class", ".jar"] or base_name in [
                "pom.xml",
                "build.gradle",
            ]:
                languages.add("java")

            # JavaScript
            elif ext in [".js", ".mjs", ".cjs", ".jsx"]:
                languages.add("javascript")

            # TypeScript
            elif ext in [".ts", ".tsx", ".cts", ".mts"]:
                languages.add("typescript")

            # PHP
            elif ext in [".php", ".phtml", ".php5", ".php7", ".phps"]:
                languages.add("php")

            # Ruby
            elif ext in [".rb", ".rake", ".gemspec"] or base_name in [
                "Gemfile",
                "Rakefile",
            ]:
                languages.add("ruby")

            # Swift
            elif ext in [".swift"]:
                languages.add("swift")

            # Kotlin
            elif ext in [".kt", ".kts"]:
                languages.add("kotlin")

            # C#
            elif ext in [".cs", ".csx"]:
                languages.add("csharp")

            # SQL
            elif ext in [".sql"]:
                languages.add("sql")

            # Shell/Bash
            elif (
                ext in [".sh", ".bash"]
                or base_name.startswith(".bash")
                or base_name.startswith(".zsh")
            ):
                languages.add("shell")

            # HTML/CSS
            elif ext in [".html", ".htm", ".xhtml"]:
                languages.add("html")
            elif ext in [".css", ".scss", ".sass", ".less"]:
                languages.add("css")

            # XML/JSON/YAML (config files)
            elif ext in [
                ".xml",
                ".xsd",
                ".dtd",
                ".tld",
                ".jsp",
                ".jspx",
                ".tag",
                ".tagx",
            ]:
                languages.add("xml")
            elif ext in [".json", ".jsonl", ".json5"]:
                languages.add("json")
            elif ext in [".yaml", ".yml"]:
                languages.add("yaml")

            # Markdown/Documentation
            elif ext in [".md", ".markdown", ".rst", ".adoc"]:
                languages.add("markdown")

            # Docker
            elif ext in [".dockerfile"] or base_name in [
                "Dockerfile",
                "docker-compose.yml",
                "docker-compose.yaml",
            ]:
                languages.add("docker")

            # Terraform
            elif ext in [".tf", ".tfvars"]:
                languages.add("terraform")

            # Makefile
            elif ext in [".mk"] or base_name == "Makefile":
                languages.add("makefile")

        # Map to our primary language groups for checklist purposes
        primary_languages = set()
        language_mapping = {
            "c_cpp": "c_cpp",
            "go": "go",
            "python": "python",
            "rust": "rust",
            "java": "java",
            "javascript": "javascript",
            "typescript": "typescript",
            "php": "php",
            "ruby": "ruby",
            "swift": "swift",
            "kotlin": "kotlin",
            "csharp": "csharp",
            "sql": "sql",
            "shell": "shell",
            "html": "html",
            "css": "css",
            "xml": "xml",
            "json": "json",
            "yaml": "yaml",
            "markdown": "docs",
            "docker": "docker",
            "terraform": "terraform",
            "makefile": "devops",
        }

        # Map detected languages to primary language groups
        for lang in languages:
            primary_lang = language_mapping.get(lang)
            if primary_lang:
                # Only keep languages we have checklists for
                if primary_lang in [
                    "c_cpp",
                    "go",
                    "python",
                    "rust",
                    "java",
                    "javascript",
                    "typescript",
                    "csharp",
                    "swift",
                    "php",
                    "shell",
                    "sql",
                    "ruby",
                    "kotlin",
                    "html",
                    "css",
                    "xml",
                    "json",
                    "yaml",
                    "docker",
                    "terraform",
                    "docs",
                    "markdown",
                    "devops",
                    "makefile",
                ]:
                    primary_languages.add(primary_lang)

        return list(primary_languages)

    def _get_language_checklist(self, language: str) -> str:
        """Get the checklist for a specific language."""
        checklist = get_language_checklist(language)
        return checklist if checklist else ""

    def execute(
        self, args: Dict[str, Any], agent: Optional["Agent"] = None
    ) -> Dict[str, Any]:
        try:
            review_type = args.get("review_type", "current").strip()
            root_dir = args.get("root_dir", ".")

            # Store current directory
            original_dir = os.getcwd()

            try:
                # Change to root_dir
                os.chdir(root_dir)

                # Variables to store file paths and diff output
                file_paths = []
                diff_output = ""

                # Build git diff command based on review type
                print("📊 正在获取代码变更...")

                if review_type == "commit":
                    if "commit_sha" not in args:
                        return {
                            "success": False,
                            "stdout": {},
                            "stderr": "commit_sha is required for commit review type",
                        }
                    commit_sha = args["commit_sha"].strip()
                    diff_cmd = f"git show {commit_sha} | cat -"

                    # Execute git command and get diff output
                    diff_output = subprocess.check_output(
                        diff_cmd, shell=True, text=True
                    )
                    if not diff_output:
                        return {
                            "success": False,
                            "stdout": {},
                            "stderr": "No changes to review",
                        }

                    # Extract changed files using git command
                    files_cmd = f"git show --name-only --pretty=format: {commit_sha} | grep -v '^$'"
                    try:
                        files_output = subprocess.check_output(
                            files_cmd, shell=True, text=True
                        )
                        file_paths = [
                            f.strip() for f in files_output.split("\n") if f.strip()
                        ]
                    except subprocess.CalledProcessError:
                        # Fallback to regex extraction if git command fails
                        file_pattern = r"diff --git a/.*?\s+b/(.*?)(\n|$)"
                        files = re.findall(file_pattern, diff_output)
                        file_paths = [match[0] for match in files]

                elif review_type == "range":
                    if "start_commit" not in args or "end_commit" not in args:
                        return {
                            "success": False,
                            "stdout": {},
                            "stderr": "start_commit and end_commit are required for range review type",
                        }
                    start_commit = args["start_commit"].strip()
                    end_commit = args["end_commit"].strip()
                    diff_cmd = f"git diff {start_commit}..{end_commit} | cat -"

                    # Execute git command and get diff output
                    diff_output = subprocess.check_output(
                        diff_cmd, shell=True, text=True
                    )
                    if not diff_output:
                        return {
                            "success": False,
                            "stdout": {},
                            "stderr": "No changes to review",
                        }

                    # Extract changed files using git command
                    files_cmd = f"git diff --name-only {start_commit}..{end_commit}"
                    try:
                        files_output = subprocess.check_output(
                            files_cmd, shell=True, text=True
                        )
                        file_paths = [
                            f.strip() for f in files_output.split("\n") if f.strip()
                        ]
                    except subprocess.CalledProcessError:
                        # Fallback to regex extraction if git command fails
                        file_pattern = r"diff --git a/.*?\s+b/(.*?)(\n|$)"
                        files = re.findall(file_pattern, diff_output)
                        file_paths = [match[0] for match in files]

                elif review_type == "file":
                    if "file_path" not in args:
                        return {
                            "success": False,
                            "stdout": {},
                            "stderr": "file_path is required for file review type",
                        }
                    file_path = args["file_path"].strip()
                    file_paths = [file_path]
                    diff_output = ReadCodeTool().execute(
                        {"files": [{"path": file_path}]}
                    )["stdout"]

                else:  # current changes
                    diff_cmd = "git diff HEAD | cat -"

                    # Execute git command and get diff output
                    diff_output = subprocess.check_output(
                        diff_cmd, shell=True, text=True
                    )
                    if not diff_output:
                        return {
                            "success": False,
                            "stdout": {},
                            "stderr": "No changes to review",
                        }

                    # Extract changed files using git command
                    files_cmd = "git diff --name-only HEAD"
                    try:
                        files_output = subprocess.check_output(
                            files_cmd, shell=True, text=True
                        )
                        file_paths = [
                            f.strip() for f in files_output.split("\n") if f.strip()
                        ]
                    except subprocess.CalledProcessError:
                        # Fallback to regex extraction if git command fails
                        file_pattern = r"diff --git a/.*?\s+b/(.*?)(\n|$)"
                        files = re.findall(file_pattern, diff_output)
                        file_paths = [match[0] for match in files]

                # Detect languages from the file paths
                detected_languages = self._detect_languages_from_files(file_paths)

                # Add review type and related information to the diff output
                review_info = f"""
----- 代码审查信息 -----
审查类型: {review_type}"""

                # Add specific information based on review type
                if review_type == "commit":
                    review_info += f"\n提交SHA: {args['commit_sha']}"
                elif review_type == "range":
                    review_info += f"\n起始提交: {args['start_commit']}\n结束提交: {args['end_commit']}"
                elif review_type == "file":
                    review_info += f"\n文件路径: {args['file_path']}"
                else:  # current changes
                    review_info += "\n当前未提交修改"

                # Add file list
                if file_paths:
                    review_info += "\n\n----- 变更文件列表 -----"
                    for i, path in enumerate(file_paths, 1):
                        review_info += f"\n{i}. {path}"

                # Add language-specific checklists
                if detected_languages:
                    review_info += "\n\n----- 检测到的编程语言 -----"
                    review_info += f"\n检测到的语言: {', '.join(detected_languages)}"

                    review_info += "\n\n----- 语言特定审查清单 -----"
                    for lang in detected_languages:
                        checklist = self._get_language_checklist(lang)
                        if checklist:
                            review_info += f"\n{checklist}"

                review_info += "\n------------------------\n\n"

                # Combine review info with diff output
                diff_output = review_info + diff_output

                PrettyOutput.print(diff_output, OutputType.CODE, lang="diff")
                print("✅ 代码变更获取完成")

                system_prompt = """<code_review_guide>
<role>
你是一位精益求精的首席代码审查专家，拥有多年企业级代码审计经验。你需要对所有代码变更进行极其全面、严谨且深入的审查，确保代码质量达到最高标准。
</role>

<tools>
# 代码审查工具选择
优先使用执行shell命令进行静态分析，而非依赖内置代码审查功能：

| 分析需求 | 首选工具 | 备选工具 |
|---------|---------|----------|
| 代码质量检查 | execute_script | - |
| 语法检查 | 语言特定lint工具 | - |
| 安全分析 | 安全扫描工具 | - |
| 代码统计 | loc | - |
</tools>

<commands>
# 推荐命令
- Python: `pylint <file_path>`, `flake8 <file_path>`, `mypy <file_path>`
- JavaScript/TypeScript: `eslint <file_path>`, `tsc --noEmit <file_path>`
- Java: `checkstyle <file_path>`, `pmd -d <file_path>`
- C/C++: `cppcheck <file_path>`, `clang-tidy <file_path>`
- Go: `golint <file_path>`, `go vet <file_path>`
- Rust: `cargo clippy`, `rustfmt --check <file_path>`
- 通用搜索：`rg "pattern" <files>` 查找特定代码模式
</commands>

<standards>
# 专家审查标准
1. 必须逐行分析每个修改文件，细致审查每一处变更，不遗漏任何细节
2. 基于坚实的证据识别问题，不做主观臆测，给出明确的问题定位和详细分析
3. 对每个问题提供完整可执行的解决方案，包括精确的改进代码
4. 确保报告条理清晰、层次分明，便于工程师快速采取行动
</standards>

<framework>
# 全面审查框架 (SCRIPPPS)

<category>
## S - 安全与风险 (Security & Risk)
- [ ] 发现所有潜在安全漏洞：注入攻击、授权缺陷、数据泄露风险
- [ ] 检查加密实现、密钥管理、敏感数据处理
- [ ] 审核权限验证逻辑、身份认证机制
- [ ] 检测OWASP Top 10安全风险和针对特定语言/框架的漏洞
</category>

<category>
## C - 正确性与完整性 (Correctness & Completeness)
- [ ] 验证业务逻辑和算法实现的准确性
- [ ] 全面检查条件边界、空值处理和异常情况
- [ ] 审核所有输入验证、参数校验和返回值处理
- [ ] 确保循环和递归的正确终止条件
- [ ] 严格检查线程安全和并发控制机制
</category>

<category>
## R - 可靠性与鲁棒性 (Reliability & Robustness)
- [ ] 评估代码在异常情况下的行为和恢复能力
- [ ] 审查错误处理、异常捕获和恢复策略
- [ ] 检查资源管理：内存、文件句柄、连接池、线程
- [ ] 评估容错设计和失败优雅降级机制
</category>

<category>
## I - 接口与集成 (Interface & Integration)
- [ ] 检查API合约遵守情况和向后兼容性
- [ ] 审核与外部系统的集成点和交互逻辑
- [ ] 验证数据格式、序列化和协议实现
- [ ] 评估系统边界处理和跨服务通信安全性
</category>

<category>
## P - 性能与效率 (Performance & Efficiency)
- [ ] 识别潜在性能瓶颈：CPU、内存、I/O、网络
- [ ] 审查数据结构选择和算法复杂度
- [ ] 检查资源密集型操作、数据库查询优化
- [ ] 评估缓存策略、批处理优化和并行处理机会
</category>

<category>
## P - 可移植性与平台适配 (Portability & Platform Compatibility)
- [ ] 检查跨平台兼容性问题和依赖项管理
- [ ] 评估配置管理和环境适配设计
- [ ] 审核国际化和本地化支持
- [ ] 验证部署和运行时环境需求
</category>

<category>
## S - 结构与可维护性 (Structure & Maintainability)
- [ ] 评估代码组织、模块划分和架构符合性
- [ ] 审查代码重复、设计模式应用和抽象水平
- [ ] 检查命名规范、代码风格和项目约定
- [ ] 评估文档完整性、注释质量和代码可读性
</category>
</framework>

<severity>
# 问题严重程度分级
- [ ] 严重 (P0): 安全漏洞、数据丢失风险、系统崩溃、功能严重缺陷
- [ ] 高危 (P1): 显著性能问题、可能导致部分功能失效、系统不稳定
- [ ] 中等 (P2): 功能局部缺陷、次优设计、明显的技术债务
- [ ] 低危 (P3): 代码风格问题、轻微优化机会、文档改进建议
</severity>

<output>
# 输出规范
针对每个文件的问题必须包含：
- [ ] 精确文件路径和问题影响范围
- [ ] 问题位置（起始行号-结束行号）
- [ ] 详尽问题描述，包括具体影响和潜在风险
- [ ] 严重程度分级（P0-P3）并说明理由
- [ ] 具体改进建议，提供完整、可执行的代码示例

所有审查发现必须：
1. 基于确凿的代码证据
2. 说明具体问题而非笼统评论
3. 提供清晰的技术原理分析
4. 给出完整的改进实施步骤
</output>

<language_specific>
# 语言特定审查
如果在审查信息中检测到了语言特定的审查清单，请按照清单中的项目进行逐一检查，并在报告中针对每个适用的清单项给出详细分析。
</language_specific>

我将分析上传的代码差异文件，进行全面的代码审查。
</code_review_guide>"""
                from jarvis.jarvis_tools.registry import ToolRegistry

                tool_registry = ToolRegistry()
                tool_registry.dont_use_tools(["code_review"])
                
                # Use the provided agent's model_group or get it from globals
                calling_agent = agent or get_agent(current_agent_name)
                model_group = None
                if calling_agent and hasattr(calling_agent, "model") and calling_agent.model:
                    model_group = calling_agent.model.model_group

                agent = Agent(
                    system_prompt=system_prompt,
                    name="Code Review Agent",
                    model_group=model_group,
                    summary_prompt=f"""<code_review_report>
<overview>
# 整体评估
[提供对整体代码质量、架构和主要关注点的简明概述，总结主要发现]
</overview>

<detailed_issues>
# 详细问题清单

<file>
## 文件: [文件路径]
[如果该文件没有发现问题，则明确说明"未发现问题"]

<issue>
### 问题 1
- **位置**: [起始行号-结束行号]
- **分类**: [使用SCRIPPPS框架中相关类别]
- **严重程度**: [P0/P1/P2/P3] - [简要说明判定理由]
- **问题描述**:
  [详细描述问题，包括技术原理和潜在影响]
- **改进建议**:
  ```
  [提供完整、可执行的代码示例，而非概念性建议]
  ```
</issue>

<issue>
### 问题 2
...
</issue>
</file>

<file>
## 文件: [文件路径2]
...
</file>
</detailed_issues>

<language_specific>
# 语言特定问题
[根据检测到的编程语言，提供针对语言特定清单中项目的分析]
</language_specific>

<best_practices>
# 最佳实践建议
[提供适用于整个代码库的改进建议和最佳实践]
</best_practices>

<summary>
# 总结
[总结主要问题和优先处理建议]
</summary>
</code_review_report>

<notes>
如果没有发现任何问题，请在REPORT标签内进行全面分析后明确说明"经过全面审查，未发现问题"并解释原因。
必须确保对所有修改的文件都进行了审查，并在报告中明确提及每个文件，即使某些文件没有发现问题。
如果检测到了特定编程语言，请参考语言特定的审查清单进行评估，并在报告中包含相关分析。
</notes>

输出格式：
{ot("REPORT")}
[在此处插入完整MARKDOWN格式的审查报告]
{ct("REPORT")}""",
                    output_handler=[tool_registry],
                    llm_type="thinking",
                    auto_complete=False,
                )

                # Determine if we need to split the diff due to size
                max_diff_size = 100 * 1024 * 1024  # Limit to 100MB

                if len(diff_output) > max_diff_size:
                    PrettyOutput.print(
                        f"代码差异内容总大小超过限制 ({len(diff_output)} > {max_diff_size} 字节)，将截断内容",
                        OutputType.WARNING,
                    )
                    diff_output = (
                        diff_output[:max_diff_size]
                        + "\n\n[diff content truncated due to size limitations...]"
                    )

                # Prepare the user prompt for code review
                user_prompt = f"""请对以下代码变更进行全面审查。

代码信息：
- 审查类型: {review_type}
- 变更文件列表: {len(file_paths)} 个文件
- 检测到的编程语言: {', '.join(detected_languages) if detected_languages else '未检测到特定语言'}

请根据SCRIPPPS框架和语言特定的审查清单进行分析，提供详细的代码审查报告。"""

                # Write the full diff output to a temporary file for uploading
                with tempfile.NamedTemporaryFile(
                    mode="w", suffix=".diff", delete=False
                ) as temp_file:
                    temp_file_path = temp_file.name
                    temp_file.write(diff_output)
                    temp_file.flush()

                try:
                    # Check if content is too large
                    is_large_content = is_context_overflow(diff_output, model_group)

                    # Upload the file to the agent's model
                    if is_large_content:
                        if not agent.model or not agent.model.support_upload_files():
                            return {
                                "success": False,
                                "stdout": "",
                                "stderr": "代码差异太大，无法处理",
                            }
                        print("📤 正在上传代码差异文件...")
                        upload_success = agent.model.upload_files([temp_file_path])
                        if upload_success:
                            print("✅ 已成功上传代码差异文件")
                        else:
                            return {
                                "success": False,
                                "stdout": "",
                                "stderr": "上传代码差异文件失败",
                            }

                    # Prepare the prompt based on upload status
                    if is_large_content:
                        # When file is uploaded, reference it in the prompt
                        complete_prompt = (
                            user_prompt
                            + f"""

我已上传了一个包含代码差异的文件。该文件包含:
- 审查类型: {review_type}
- 变更文件数量: {len(file_paths)} 个文件
- 检测到的编程语言: {', '.join(detected_languages) if detected_languages else '未检测到特定语言'}

请基于上传的代码差异文件进行全面审查，并生成详细的代码审查报告。"""
                        )
                        # Run the agent with the prompt
                        result = agent.run(complete_prompt)
                    else:
                        complete_prompt = (
                            user_prompt
                            + "\n\n代码差异内容:\n```diff\n"
                            + diff_output
                            + "\n```"
                        )
                        result = agent.run(complete_prompt)
                finally:
                    # Clean up the temporary file
                    if os.path.exists(temp_file_path):
                        try:
                            os.unlink(temp_file_path)
                        except Exception:
                            PrettyOutput.print(
                                f"临时文件 {temp_file_path} 未能删除",
                                OutputType.WARNING,
                            )

                return {"success": True, "stdout": result, "stderr": ""}
            finally:
                # Always restore original directory
                os.chdir(original_dir)

        except Exception as e:
            return {
                "success": False,
                "stdout": {},
                "stderr": f"Review failed: {str(e)}",
            }


def extract_code_report(result: str) -> str:
    sm = re.search(ot("REPORT") + r"\n(.*?)\n" + ct("REPORT"), result, re.DOTALL)
    if sm:
        return sm.group(1)
    return result


@app.command("commit")
def review_commit(
    commit: str = typer.Argument(..., help="Commit SHA to review"),
    root_dir: str = typer.Option(".", "--root-dir", help="Root directory of the codebase"),
):
    """Review specific commit"""
    tool = CodeReviewTool()
    tool_args = {"review_type": "commit", "commit_sha": commit, "root_dir": root_dir}
    result = tool.execute(tool_args)
    if result["success"]:
        PrettyOutput.section("自动代码审查结果:", OutputType.SUCCESS)
        report = extract_code_report(result["stdout"])
        PrettyOutput.print(report, OutputType.SUCCESS, lang="markdown")
    else:
        PrettyOutput.print(result["stderr"], OutputType.WARNING)


@app.command("current")
def review_current(
    root_dir: str = typer.Option(".", "--root-dir", help="Root directory of the codebase"),
):
    """Review current changes"""
    tool = CodeReviewTool()
    tool_args = {"review_type": "current", "root_dir": root_dir}
    result = tool.execute(tool_args)
    if result["success"]:
        PrettyOutput.section("自动代码审查结果:", OutputType.SUCCESS)
        report = extract_code_report(result["stdout"])
        PrettyOutput.print(report, OutputType.SUCCESS, lang="markdown")
    else:
        PrettyOutput.print(result["stderr"], OutputType.WARNING)


@app.command("range")
def review_range(
    start_commit: str = typer.Argument(..., help="Start commit SHA"),
    end_commit: str = typer.Argument(..., help="End commit SHA"),
    root_dir: str = typer.Option(".", "--root-dir", help="Root directory of the codebase"),
):
    """Review commit range"""
    tool = CodeReviewTool()
    tool_args = {
        "review_type": "range",
        "start_commit": start_commit,
        "end_commit": end_commit,
        "root_dir": root_dir,
    }
    result = tool.execute(tool_args)
    if result["success"]:
        PrettyOutput.section("自动代码审查结果:", OutputType.SUCCESS)
        report = extract_code_report(result["stdout"])
        PrettyOutput.print(report, OutputType.SUCCESS, lang="markdown")
    else:
        PrettyOutput.print(result["stderr"], OutputType.WARNING)


@app.command("file")
def review_file(
    file: str = typer.Argument(..., help="File path to review"),
    root_dir: str = typer.Option(".", "--root-dir", help="Root directory of the codebase"),
):
    """Review specific file"""
    tool = CodeReviewTool()
    tool_args = {"review_type": "file", "file_path": file, "root_dir": root_dir}
    result = tool.execute(tool_args)
    if result["success"]:
        PrettyOutput.section("自动代码审查结果:", OutputType.SUCCESS)
        report = extract_code_report(result["stdout"])
        PrettyOutput.print(report, OutputType.SUCCESS, lang="markdown")
    else:
        PrettyOutput.print(result["stderr"], OutputType.WARNING)


def cli():
    """Typer application entry point"""
    init_env("欢迎使用 Jarvis-CodeReview，您的代码审查助手已准备就绪！")
    app()


def main():
    """Main entry point for the script"""
    cli()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
方法论导入导出命令行工具

功能：
- 导入方法论文件（合并策略）
- 导出当前方法论
- 列出所有方法论
"""

import hashlib
import json
import os
import sys
from typing import Optional

import typer
import yaml  # type: ignore

from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_utils.methodology import (
    _get_methodology_directory,
    _load_all_methodologies,
)
from jarvis.jarvis_utils.output import OutputType, PrettyOutput

app = typer.Typer(help="方法论管理工具")


@app.command("import")
def import_methodology(
    input_file: str = typer.Argument(..., help="要导入的方法论文件路径")
):
    """导入方法论文件（合并策略）"""
    try:
        # 加载现有方法论
        existing_methodologies = _load_all_methodologies()

        # 加载要导入的方法论
        with open(input_file, "r", encoding="utf-8") as f:
            import_data = json.load(f)

        # 合并方法论（新数据会覆盖旧数据）
        merged_data = {**existing_methodologies, **import_data}

        # 保存合并后的方法论
        methodology_dir = _get_methodology_directory()
        for problem_type, content in merged_data.items():
            safe_filename = hashlib.md5(problem_type.encode("utf-8")).hexdigest()
            file_path = os.path.join(methodology_dir, f"{safe_filename}.json")

            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(
                    {"problem_type": problem_type, "content": content},
                    f,
                    ensure_ascii=False,
                    indent=2,
                )

        PrettyOutput.print(
            f"成功导入 {len(import_data)} 个方法论（总计 {len(merged_data)} 个）",
            OutputType.SUCCESS,
        )
    except (json.JSONDecodeError, OSError) as e:
        PrettyOutput.print(f"导入失败: {str(e)}", OutputType.ERROR)
        raise typer.Exit(code=1)


@app.command("export")
def export_methodology(
    output_file: str = typer.Argument(..., help="导出文件路径")
):
    """导出当前方法论到单个文件"""
    try:
        methodologies = _load_all_methodologies()

        with open(output_file, "w", encoding="utf-8") as f:
            json.dump(methodologies, f, ensure_ascii=False, indent=2)

        PrettyOutput.print(
            f"成功导出 {len(methodologies)} 个方法论到 {output_file}",
            OutputType.SUCCESS,
        )
    except (OSError, TypeError) as e:
        PrettyOutput.print(f"导出失败: {str(e)}", OutputType.ERROR)
        raise typer.Exit(code=1)


@app.command("list")
def list_methodologies():
    """列出所有方法论"""
    try:
        methodologies = _load_all_methodologies()

        if not methodologies:
            PrettyOutput.print("没有找到方法论", OutputType.INFO)
            return

        PrettyOutput.print("可用方法论:", OutputType.INFO)
        for i, (problem_type, _) in enumerate(methodologies.items(), 1):
            PrettyOutput.print(f"{i}. {problem_type}", OutputType.INFO)
    except (OSError, json.JSONDecodeError) as e:
        PrettyOutput.print(f"列出方法论失败: {str(e)}", OutputType.ERROR)
        raise typer.Exit(code=1)


@app.command("extract")
def extract_methodology(
    input_file: str = typer.Argument(..., help="要提取方法论的文本文件路径")
):
    """从文本文件中提取方法论"""
    try:
        # 读取文本文件内容
        with open(input_file, "r", encoding="utf-8") as f:
            text_content = f.read()

        # 获取平台实例
        platform = PlatformRegistry().get_normal_platform()

        # 构建提取提示
        prompt = f"""请从以下文本中提取方法论：
        
{text_content}

请按以下格式返回结果：
<methodologies>
- problem_type: [问题类型1]
  content: |2
    [多行方法论内容1]
- problem_type: [问题类型2]
  content: |2
    [多行方法论内容2]
</methodologies>

要求：
1. 方法论应聚焦于通用且可重复的解决方案流程
2. 方法论应该具备足够的通用性，可应用于同类问题
3. 方法论内容应包含：
   - 问题重述: 简明扼要的问题归纳
   - 最优解决方案: 经过验证的解决方案
   - 注意事项: 执行中可能遇到的问题
   - 可选步骤: 多种解决路径和适用场景
4. 在<methodologies>标签中直接使用YAML列表
5. 确保YAML缩进正确
6. 内容字段使用|保留多行格式
"""

        # 调用大模型平台提取方法论
        print("📥 正在提取方法论...")
        try:
            response = platform.chat_until_success(prompt)
        except Exception as e:
            print("❌ 提取失败")
            PrettyOutput.print(f"提取方法论失败: {str(e)}", OutputType.ERROR)
            raise typer.Exit(code=1)

        # 提取YAML部分
        methodologies_start = response.find("<methodologies>") + len("<methodologies>")
        methodologies_end = response.find("</methodologies>")
        if methodologies_start == -1 or methodologies_end == -1:
            print("❌ 响应格式无效")
            PrettyOutput.print(
                "大模型未返回有效的<methodologies>格式", OutputType.ERROR
            )
            raise typer.Exit(code=1)

        yaml_content = response[methodologies_start:methodologies_end].strip()

        try:
            data = yaml.safe_load(yaml_content)
            extracted_methodologies = {
                item["problem_type"]: item["content"] for item in data
            }
        except (yaml.YAMLError, KeyError, TypeError) as e:
            print("❌ YAML解析失败")
            PrettyOutput.print(f"YAML解析错误: {str(e)}", OutputType.ERROR)
            raise typer.Exit(code=1)

        if not extracted_methodologies:
            print("❌ 未提取到有效方法论")
            return
        print("✅ 提取到有效方法论")

        # 加载现有方法论
        existing_methodologies = _load_all_methodologies()

        # 合并方法论（新数据会覆盖旧数据）
        merged_data = {**existing_methodologies, **extracted_methodologies}

        # 保存合并后的方法论
        methodology_dir = _get_methodology_directory()
        for problem_type, content in merged_data.items():
            safe_filename = hashlib.md5(problem_type.encode("utf-8")).hexdigest()
            file_path = os.path.join(methodology_dir, f"{safe_filename}.json")

            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(
                    {"problem_type": problem_type, "content": content},
                    f,
                    ensure_ascii=False,
                    indent=2,
                )

        PrettyOutput.print(
            f"成功从文件提取 {len(extracted_methodologies)} 个方法论（总计 {len(merged_data)} 个）",
            OutputType.SUCCESS,
        )
    except Exception as e:
        PrettyOutput.print(f"提取失败: {str(e)}", OutputType.ERROR)
        raise typer.Exit(code=1)


@app.command("extract-url")
def extract_methodology_from_url(url: str = typer.Argument(..., help="要提取方法论的URL")):
    """从URL提取方法论"""
    try:
        # 获取平台实例
        platform = PlatformRegistry().get_normal_platform()

        platform.web = True

        # 构建提取提示
        prompt = f"""请从以下URL内容中提取方法论：
        
{url}

请按以下格式返回结果：
<methodologies>
- problem_type: [问题类型1]
  content: |2
    [多行方法论内容1]
- problem_type: [问题类型2]
  content: |2
    [多行方法论内容2]
</methodologies>

要求：
1. 方法论应聚焦于通用且可重复的解决方案流程
2. 方法论应该具备足够的通用性，可应用于同类问题
3. 方法论内容应包含：
   - 问题重述: 简明扼要的问题归纳
   - 最优解决方案: 经过验证的解决方案
   - 注意事项: 执行中可能遇到的问题
   - 可选步骤: 多种解决路径和适用场景
4. 在<methodologies>标签中直接使用YAML列表
5. 确保YAML缩进正确
6. 内容字段使用|保留多行格式
"""
        # 调用大模型平台提取方法论
        print("📥 正在从URL提取方法论...")
        try:
            response = platform.chat_until_success(prompt)
        except Exception as e:
            print("❌ 提取失败")
            PrettyOutput.print(f"提取方法论失败: {str(e)}", OutputType.ERROR)
            raise typer.Exit(code=1)

        # 提取YAML部分
        methodologies_start = response.find("<methodologies>") + len("<methodologies>")
        methodologies_end = response.find("</methodologies>")
        if methodologies_start == -1 or methodologies_end == -1:
            print("❌ 响应格式无效")
            PrettyOutput.print(
                "大模型未返回有效的<methodologies>格式", OutputType.ERROR
            )
            raise typer.Exit(code=1)

        yaml_content = response[methodologies_start:methodologies_end].strip()

        try:
            data = yaml.safe_load(yaml_content)
            extracted_methodologies = {
                item["problem_type"]: item["content"] for item in data
            }
        except (yaml.YAMLError, KeyError, TypeError) as e:
            print("❌ YAML解析失败")
            PrettyOutput.print(f"YAML解析错误: {str(e)}", OutputType.ERROR)
            raise typer.Exit(code=1)

        if not extracted_methodologies:
            print("❌ 未提取到有效方法论")
            return
        print("✅ 提取到有效方法论")

        # 加载现有方法论
        existing_methodologies = _load_all_methodologies()

        # 合并方法论（新数据会覆盖旧数据）
        merged_data = {**existing_methodologies, **extracted_methodologies}

        # 保存合并后的方法论
        methodology_dir = _get_methodology_directory()
        for problem_type, content in merged_data.items():
            safe_filename = hashlib.md5(problem_type.encode("utf-8")).hexdigest()
            file_path = os.path.join(methodology_dir, f"{safe_filename}.json")

            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(
                    {"problem_type": problem_type, "content": content},
                    f,
                    ensure_ascii=False,
                    indent=2,
                )

        PrettyOutput.print(
            f"成功从URL提取 {len(extracted_methodologies)} 个方法论（总计 {len(merged_data)} 个）",
            OutputType.SUCCESS,
        )
    except Exception as e:
        PrettyOutput.print(f"从URL提取失败: {str(e)}", OutputType.ERROR)
        raise typer.Exit(code=1)


def main() -> None:
    """Application entry point"""
    app()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
方法论管理模块
该模块提供了加载和搜索方法论的实用工具。
包含以下功能：
- 加载和处理方法论数据
- 生成方法论临时文件
- 上传方法论文件到大模型
"""
import json
import os
import tempfile
from pathlib import Path
from typing import Any, Dict, List, Optional

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_utils.config import get_data_dir, get_methodology_dirs
from jarvis.jarvis_utils.globals import get_agent, current_agent_name
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.utils import is_context_overflow, daily_check_git_updates

def _get_methodology_directory() -> str:
    """
    获取方法论目录路径，如果不存在则创建

    返回：
        str: 方法论目录的路径
    """
    methodology_dir = os.path.join(get_data_dir(), "methodologies")
    if not os.path.exists(methodology_dir):
        try:
            os.makedirs(methodology_dir, exist_ok=True)
        except Exception as e:
            PrettyOutput.print(f"创建方法论目录失败: {str(e)}", OutputType.ERROR)
    return methodology_dir


def _load_all_methodologies() -> Dict[str, str]:
    """
    从默认目录和配置的外部目录加载所有方法论文件。

    返回：
        Dict[str, str]: 方法论字典，键为问题类型，值为方法论内容。
    """
    all_methodologies = {}
    methodology_dirs = [_get_methodology_directory()] + get_methodology_dirs()

    # --- 全局每日更新检查 ---
    daily_check_git_updates(methodology_dirs, "methodologies")

    import glob

    for directory in set(methodology_dirs):  # Use set to avoid duplicates
        if not os.path.isdir(directory):
            PrettyOutput.print(f"警告: 方法论目录不存在或不是一个目录: {directory}", OutputType.WARNING)
            continue

        for filepath in glob.glob(os.path.join(directory, "*.json")):
            try:
                with open(filepath, "r", encoding="utf-8", errors="ignore") as f:
                    methodology = json.load(f)
                    problem_type = methodology.get("problem_type", "")
                    content = methodology.get("content", "")
                    if problem_type and content:
                        if problem_type in all_methodologies:
                            PrettyOutput.print(f"警告: 方法论 '{problem_type}' 被 '{filepath}' 覆盖。", OutputType.WARNING)
                        all_methodologies[problem_type] = content
            except Exception as e:
                filename = os.path.basename(filepath)
                PrettyOutput.print(
                    f"加载方法论文件 {filename} 失败: {str(e)}", OutputType.WARNING
                )

    return all_methodologies


def _create_methodology_temp_file(methodologies: Dict[str, str]) -> Optional[str]:
    """
    创建包含所有方法论的临时文件

    参数：
        methodologies: 方法论字典，键为问题类型，值为方法论内容

    返回：
        Optional[str]: 临时文件路径，如果创建失败则返回None
    """
    if not methodologies:
        return None

    try:
        # 创建临时文件
        fd, temp_path = tempfile.mkstemp(suffix=".md", prefix="methodologies_")
        os.close(fd)

        # 写入方法论内容
        with open(temp_path, "w", encoding="utf-8") as f:
            f.write("# 方法论集合\n\n")
            for problem_type, content in methodologies.items():
                f.write(f"## {problem_type}\n\n")
                f.write(f"{content}\n\n")
                f.write("---\n\n")
            f.flush()

        return temp_path
    except Exception as e:
        PrettyOutput.print(f"创建方法论临时文件失败: {str(e)}", OutputType.ERROR)
        return None


def upload_methodology(platform: BasePlatform, other_files: List[str] = []) -> bool:
    """
    上传方法论文件到指定平台

    参数：
        platform: 平台实例，需实现upload_files方法

    返回：
        bool: 上传是否成功
    """
    methodology_dir = _get_methodology_directory()
    if not os.path.exists(methodology_dir):
        PrettyOutput.print("方法论文档不存在", OutputType.WARNING)
        return False

    methodologies = _load_all_methodologies()
    if not methodologies:
        PrettyOutput.print("没有可用的方法论文档", OutputType.WARNING)
        return False

    temp_file_path = _create_methodology_temp_file(methodologies)
    if not temp_file_path:
        return False

    try:
        return platform.upload_files([temp_file_path, *other_files])

    finally:
        if temp_file_path and os.path.exists(temp_file_path):
            try:
                os.remove(temp_file_path)
            except Exception:
                pass


def load_methodology(user_input: str, tool_registery: Optional[Any] = None) -> str:
    """
    加载方法论并上传到大模型。

    参数：
        user_input: 用户输入文本，用于提示大模型

    返回：
        str: 相关的方法论提示，如果未找到方法论则返回空字符串
    """

    prompt = tool_registery.prompt() if tool_registery else ""

    # 获取方法论目录
    methodology_dir = _get_methodology_directory()
    if not os.path.exists(methodology_dir):
        return ""

    try:
        # 加载所有方法论
        print(f"📁 加载方法论文件...")
        methodologies = _load_all_methodologies()
        if not methodologies:
            print(f"❌ 没有找到方法论文件")
            return ""
        print(f"✅ 加载方法论文件完成 (共 {len(methodologies)} 个)")

        # 获取当前平台
        agent = get_agent(current_agent_name)
        if agent:
            platform = agent.model
            model_group = agent.model.model_group
        else:
            platform = PlatformRegistry().get_normal_platform()
            model_group = None
        platform.set_suppress_output(False)
        if not platform:
            return ""

        # 构建基础提示信息
        base_prompt = f"""以下是所有可用的方法论内容：

"""
        # 构建完整内容
        full_content = base_prompt
        for problem_type, content in methodologies.items():
            full_content += f"## {problem_type}\n\n{content}\n\n---\n\n"

        full_content += f"以下是所有可用的工具内容：\n\n"
        full_content += prompt

        # 添加用户输入和输出要求
        full_content += f"""
请根据以上方法论和可调用的工具内容，规划/总结出以下用户需求的执行步骤: {user_input}

请按以下格式回复：
### 与该任务/需求相关的方法论
1. [方法论名字]
2. [方法论名字]
### 根据以上方法论，规划/总结出执行步骤
1. [步骤1]
2. [步骤2]
3. [步骤3]

如果没有匹配的方法论，请输出：没有历史方法论可参考
除以上要求外，不要输出任何内容
"""

        # 检查内容是否过大
        is_large_content = is_context_overflow(full_content, model_group)
        temp_file_path = None

        try:
            if is_large_content:
                # 创建临时文件
                print(f"📝 创建方法论临时文件...")
                temp_file_path = _create_methodology_temp_file(methodologies)
                if not temp_file_path:
                    print(f"❌ 创建方法论临时文件失败")
                    return ""
                print(f"✅ 创建方法论临时文件完成")

                # 尝试上传文件
                upload_success = platform.upload_files([temp_file_path])

                if upload_success:
                    # 使用上传的文件生成摘要
                    return platform.chat_until_success(
                        base_prompt
                        + f"""
请根据已上传的方法论和可调用的工具文件内容，规划/总结出以下用户需求的执行步骤: {user_input}

请按以下格式回复：
### 与该任务/需求相关的方法论
1. [方法论名字]
2. [方法论名字]
### 根据以上方法论，规划/总结出执行步骤
1. [步骤1]
2. [步骤2]
3. [步骤3]

如果没有匹配的方法论，请输出：没有历史方法论可参考
除以上要求外，不要输出任何内容
"""
                    )
                else:
                    return "没有历史方法论可参考"
            # 如果内容不大或上传失败，直接使用chat_until_success
            return platform.chat_until_success(full_content)

        finally:
            # 清理临时文件
            if temp_file_path and os.path.exists(temp_file_path):
                try:
                    os.remove(temp_file_path)
                except Exception:
                    pass

    except Exception as e:
        PrettyOutput.print(f"加载方法论失败: {str(e)}", OutputType.ERROR)
        return ""



# -*- coding: utf-8 -*-
"""
Kotlin-specific code review checklist.
"""

CHECKLIST = """
## Kotlin 代码审查检查清单

### 代码风格和约定
- [ ] 代码是否遵循 Kotlin 官方编码规范
- [ ] 是否使用工具（如 ktlint、detekt）进行代码风格检查
- [ ] 格式化是否一致（缩进、空格、换行等）
- [ ] 命名是否符合 Kotlin 约定（驼峰命名法，常量使用大写）
- [ ] 是否避免了魔法数字和硬编码字符串
- [ ] 是否避免使用 !!（非空断言）操作符
- [ ] 注释是否清晰且及时更新
- [ ] 文件组织是否合理（每个文件一个类，相关功能分组）

### Kotlin 特性使用
- [ ] 是否恰当使用扩展函数和属性
- [ ] 是否有效利用 null 安全操作符（?、?.、?:、!!）
- [ ] 是否合理使用数据类（data class）
- [ ] 是否合理使用密封类（sealed class）
- [ ] 是否恰当使用伴生对象（companion object）
- [ ] 是否合理使用对象声明（object）
- [ ] 是否合理使用高阶函数和 lambda 表达式
- [ ] 是否恰当使用委托属性（properties delegation）

### 面向对象设计
- [ ] 类的设计是否遵循单一职责原则
- [ ] 是否恰当使用继承和接口
- [ ] 是否避免过度使用继承
- [ ] 是否合理使用抽象类和接口
- [ ] 是否给类和方法提供适当的可见性修饰符
- [ ] 构造函数设计是否合理
- [ ] 是否遵循开闭原则和其他 SOLID 原则
- [ ] 是否使用组合而非继承（适当情况下）

### 并发和异步编程
- [ ] 是否正确使用协程（coroutines）
- [ ] 是否合理使用挂起函数（suspend functions）
- [ ] 是否恰当处理协程的取消和异常
- [ ] 是否合理使用协程上下文（CoroutineContext）
- [ ] 是否合理选择适当的调度器（Dispatchers）
- [ ] 是否避免共享可变状态或正确保护它
- [ ] 是否考虑线程安全问题
- [ ] 是否使用了适当的并发原语（如 Mutex）

### Android 特定检查 (如适用)
- [ ] 是否正确管理生命周期
- [ ] 是否避免内存泄漏（如 Context 引用）
- [ ] UI 组件是否遵循 Material Design 原则
- [ ] 是否有效使用 ViewModel 和 LiveData
- [ ] 是否恰当使用 Android KTX 扩展
- [ ] 是否适当配置了线程（主线程/后台线程）
- [ ] 是否遵循官方 Android 架构指南
- [ ] 是否妥善处理配置更改和进程死亡

### 性能优化
- [ ] 是否避免了昂贵的操作（如在循环中创建对象）
- [ ] 集合操作是否高效（适当使用序列进行大量元素的处理）
- [ ] 是否避免了装箱和拆箱开销
- [ ] 是否优化了闭包和内联函数使用
- [ ] 是否避免在关键路径上分配过多对象
- [ ] 是否考虑了惰性初始化（lazy）以优化资源使用
- [ ] 是否合理缓存计算结果
- [ ] 是否使用了适当的数据结构和算法

### 依赖管理
- [ ] 依赖是否声明精确的版本
- [ ] 是否避免使用过时或不安全的依赖
- [ ] 是否最小化依赖数量
- [ ] 是否定期更新依赖以修复安全漏洞
- [ ] 是否处理了依赖冲突
- [ ] 是否区分了开发和生产依赖
- [ ] 是否使用适当的构建系统（Gradle）配置
- [ ] 是否合理组织和模块化项目结构

### 错误处理
- [ ] 是否使用了合适的错误处理策略
- [ ] 是否考虑使用 Result 类型代替异常
- [ ] 是否创建了自定义异常类型
- [ ] 是否在适当的级别捕获和处理异常
- [ ] 是否提供有意义的错误消息
- [ ] 是否处理了所有可能的错误情况
- [ ] 是否记录了关键异常
- [ ] try-catch 块是否尽可能小而聚焦

### 测试
- [ ] 单元测试是否充分且有意义
- [ ] 是否使用了适当的测试框架（JUnit、Mockito、MockK）
- [ ] 测试是否涵盖主要功能和边界条件
- [ ] UI 测试是否覆盖关键用户流程
- [ ] 测试代码组织是否清晰
- [ ] 测试是否避免了不必要的复杂性
- [ ] 是否进行了性能和内存使用测试（适用时）
- [ ] 测试是否快速且可靠（无随机失败）

### 文档
- [ ] 是否使用 KDoc 注释公共 API
- [ ] 是否有清晰的 README 和项目文档
- [ ] 是否记录了复杂逻辑
- [ ] 文档是否与代码同步更新
- [ ] 是否记录了性能注意事项和限制
- [ ] API 参数和返回值是否有文档说明
- [ ] 是否包含了示例代码或用例
- [ ] 是否维护了 CHANGELOG 记录
"""



# -*- coding: utf-8 -*-
import re
from abc import ABC, abstractmethod
from types import TracebackType
from typing import Generator, List, Optional, Tuple, Type

from typing_extensions import Self

from rich import box  # type: ignore
from rich.live import Live  # type: ignore
from rich.panel import Panel  # type: ignore
from rich.text import Text  # type: ignore

from jarvis.jarvis_utils.config import (
    get_max_input_token_count,
    get_pretty_output,
    is_print_prompt,
)
from jarvis.jarvis_utils.embedding import split_text_into_chunks
from jarvis.jarvis_utils.globals import set_in_chat
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot
from jarvis.jarvis_utils.utils import get_context_token_count, while_success, while_true


class BasePlatform(ABC):
    """Base class for large language models"""

    def __init__(self):
        """Initialize model"""
        self.suppress_output = True  # 添加输出控制标志
        self.web = False  # 添加web属性，默认false
        self._saved = False
        self.model_group: Optional[str] = None

    def __enter__(self) -> Self:
        """Enter context manager"""
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        """Exit context manager"""
        if not self._saved:
            self.delete_chat()

    @abstractmethod
    def set_model_name(self, model_name: str):
        """Set model name"""
        raise NotImplementedError("set_model_name is not implemented")

    def reset(self):
        """Reset model"""
        self.delete_chat()

    @abstractmethod
    def chat(self, message: str) -> Generator[str, None, None]:
        """Execute conversation"""
        raise NotImplementedError("chat is not implemented")

    @abstractmethod
    def upload_files(self, file_list: List[str]) -> bool:
        raise NotImplementedError("upload_files is not implemented")

    @abstractmethod
    def support_upload_files(self) -> bool:
        """Check if platform supports upload files"""
        return False

    def _chat(self, message: str):
        import time

        start_time = time.time()

        input_token_count = get_context_token_count(message)

        if input_token_count > get_max_input_token_count(self.model_group):
            max_chunk_size = (
                get_max_input_token_count(self.model_group) - 1024
            )  # 留出一些余量
            min_chunk_size = get_max_input_token_count(self.model_group) - 2048
            inputs = split_text_into_chunks(message, max_chunk_size, min_chunk_size)
            print("📤 正在提交长上下文...")
            prefix_prompt = f"""
            我将分多次提供大量内容，在我明确告诉你内容已经全部提供完毕之前，每次仅需要输出"已收到"，明白请输出"开始接收输入"。
            """
            while_true(lambda: while_success(lambda: self.chat(prefix_prompt), 5), 5)
            submit_count = 0
            length = 0
            response = ""
            for input in inputs:
                submit_count += 1
                length += len(input)
                print(
                    f"📤 正在提交第{submit_count}部分（共{len(inputs)}部分({length}/{len(message)})）"
                )

                response += "\n"
                for trunk in while_true(
                    lambda: while_success(
                        lambda: self.chat(
                            f"<part_content>{input}</part_content>\n\n请返回<已收到>，不需要返回其他任何内容"
                        ),
                        5,
                    ),
                    5,
                ):
                    response += trunk

                print(f"📤 提交第{submit_count}部分完成，当前进度：{length}/{len(message)}")
            print("✅ 提交完成")
            response += "\n" + while_true(
                lambda: while_success(lambda: self._chat("内容已经全部提供完毕，请根据内容继续"), 5),
                5,
            )
        else:
            response = ""

            text_content = Text()
            panel = Panel(
                text_content,
                title=f"[bold cyan]{self.name()}[/bold cyan]",
                subtitle="[dim]思考中...[/dim]",
                border_style="bright_blue",
                box=box.ROUNDED,
            )

            if not self.suppress_output:
                if get_pretty_output():
                    with Live(panel, refresh_per_second=10, transient=False) as live:
                        for s in self.chat(message):
                            response += s
                            text_content.append(s, style="bright_white")
                            panel.subtitle = "[yellow]正在回答...[/yellow]"
                            live.update(panel)
                        end_time = time.time()
                        duration = end_time - start_time
                        char_count = len(response)
                        # Calculate token count and tokens per second
                        try:
                            token_count = get_context_token_count(response)
                            tokens_per_second = (
                                token_count / duration if duration > 0 else 0
                            )
                        except Exception as e:
                            PrettyOutput.print(
                                f"Tokenization failed: {str(e)}", OutputType.WARNING
                            )
                            token_count = 0
                            tokens_per_second = 0
                        panel.subtitle = f"[bold green]✓ 对话完成耗时: {duration:.2f}秒, 输入字符数: {len(message)}, 输入Token数量: {input_token_count}, 输出字符数: {char_count}, 输出Token数量: {token_count}, 每秒Token数量: {tokens_per_second:.2f}[/bold green]"
                        live.update(panel)
                else:
                    for s in self.chat(message):
                        print(s, end="", flush=True)
                        response += s
                    print()
            else:
                for s in self.chat(message):
                    response += s
        # Keep original think tag handling
        response = re.sub(
            ot("think") + r".*?" + ct("think"), "", response, flags=re.DOTALL
        )
        response = re.sub(
            ot("thinking") + r".*?" + ct("thinking"), "", response, flags=re.DOTALL
        )
        return response

    def chat_until_success(self, message: str) -> str:
        """Chat with model until successful response"""
        try:
            set_in_chat(True)
            if not self.suppress_output and is_print_prompt():
                PrettyOutput.print(f"{message}", OutputType.USER)
            result: str = while_true(
                lambda: while_success(lambda: self._chat(message), 5), 5
            )
            from jarvis.jarvis_utils.globals import set_last_message

            set_last_message(result)
            return result
        finally:
            set_in_chat(False)

    @abstractmethod
    def name(self) -> str:
        """Model name"""
        raise NotImplementedError("name is not implemented")

    @classmethod
    @abstractmethod
    def platform_name(cls) -> str:
        """Platform name"""
        raise NotImplementedError("platform_name is not implemented")

    @abstractmethod
    def delete_chat(self) -> bool:
        """Delete chat"""
        raise NotImplementedError("delete_chat is not implemented")

    @abstractmethod
    def save(self, file_path: str) -> bool:
        """Save chat session to a file.

        Note:
            Implementations of this method should set `self._saved = True` upon successful saving
            to prevent the session from being deleted on object destruction.

        Args:
            file_path: The path to save the session file.

        Returns:
            True if saving is successful, False otherwise.
        """
        raise NotImplementedError("save is not implemented")

    @abstractmethod
    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file.

        Args:
            file_path: The path to restore the session file from.

        Returns:
            True if restoring is successful, False otherwise.
        """
        raise NotImplementedError("restore is not implemented")

    @abstractmethod
    def set_system_prompt(self, message: str):
        """Set system message"""
        raise NotImplementedError("set_system_prompt is not implemented")

    @abstractmethod
    def get_model_list(self) -> List[Tuple[str, str]]:
        """Get model list"""
        raise NotImplementedError("get_model_list is not implemented")

    def set_suppress_output(self, suppress: bool):
        """Set whether to suppress output"""
        self.suppress_output = suppress

    def set_model_group(self, model_group: Optional[str]):
        """Set model group"""
        self.model_group = model_group

    def set_web(self, web: bool):
        """Set web flag"""
        self.web = web

    @abstractmethod
    def support_web(self) -> bool:
        """Check if platform supports web functionality"""
        return False
